#include <sys/time.h>

#include <qlayout.h>
#include <qpainter.h>
#include <qtimer.h>
#include <iostream>
#include <qfileinfo.h>

#include "dc_settings.h"

#include "transfer_widget.h"
#include "transfer_view.h"


#include "debugdlg.h"

// Least interval between requesting gdl-list updates.
#define GDL_UPDATE_INTERVAL (1000)

// Least interval between emitting 'start gdl' and 'add dl source' signals.
#define SIGNAL_EMIT_INTERVAL (5000)

transfer_widget::transfer_widget(QWidget *parent, const char* name) : QWidget(parent, name)
{
  QGridLayout* layout = new QGridLayout( this, 1, 1 );
  tabs = new QTabWidget( this, "Transfer tabs" );
      
  // List views
  lv[T_DL] = new QListView(this,"Downloads list");
  lv[T_DL]->setAllColumnsShowFocus(true);
  lv[T_DL]->setRootIsDecorated(true);
  
  lv[T_DL]->addColumn("File");
  lv[T_DL]->addColumn("Size");
  lv[T_DL]->addColumn("User");
  lv[T_DL]->addColumn("Status");
  lv[T_DL]->addColumn("Remaining");
  lv[T_DL]->addColumn("Transfer Rate");
  lv[T_DL]->addColumn("Time Left");

  lv[T_Q] = new QListView(this,"Queue list");
  lv[T_Q]->setAllColumnsShowFocus(true);
  lv[T_Q]->setRootIsDecorated(true);

  lv[T_Q]->addColumn("File");
  lv[T_Q]->addColumn("Size");
  lv[T_Q]->addColumn("User");
  lv[T_Q]->addColumn("Status");

  lv[T_UL] = new QListView(this,"Uploads list");
  lv[T_UL]->setAllColumnsShowFocus(true);
  lv[T_UL]->setRootIsDecorated(false);

  lv[T_UL]->addColumn("File");
  lv[T_UL]->addColumn("Size");
  lv[T_UL]->addColumn("User");
  lv[T_UL]->addColumn("Status");
  lv[T_UL]->addColumn("Remaining");
  lv[T_UL]->addColumn("Transfer Rate");
  lv[T_UL]->addColumn("Time Left");

  lv[T_H] = new QListView(this,"History list");
  lv[T_H]->setAllColumnsShowFocus(true);
  lv[T_H]->setRootIsDecorated(true);

  lv[T_H]->addColumn("File");
  lv[T_H]->addColumn("#Files");
  lv[T_H]->addColumn("Size");
  lv[T_H]->addColumn("User");

  // History's upload & download root items.
  history_dl = new QListViewItem(lv[T_H], "Downloaded", "0", "0 Byte", "");
  history_dl->setExpandable(true);
  history_ul = new QListViewItem(lv[T_H], "Uploaded", "0", "0 Byte", "");
  history_ul->setExpandable(true);

  // Tabs.  Make sure the order is same as in enum T_TABS, or menus will not work.
  tabs->addTab( lv[T_DL], "D&ownloads");
  tabs->addTab( lv[T_Q],  "&Queue");
  tabs->addTab( lv[T_UL], "&Uploads");
  tabs->addTab( lv[T_H],  "&History");
  tabs->setCurrentPage(T_Q);
  layout->addWidget(tabs,0,0);
  
  // Popup menus & actions

  //text, menuText, QKeySequence accel, parent, name 
  menu[T_DL] = new QPopupMenu(this, "DownloadPopupMenu");
  menu[T_DL]->insertItem("&Abort download", PM_CloseGDL);
  menu[T_DL]->insertSeparator();
  menu[T_DL]->insertItem("&Close connection", PM_CloseDL);
  menu[T_DL]->insertItem("Get user's &file list", PM_GetFileList);
  menu[T_Q] = new QPopupMenu(this, "QueuePopupMenu");
  menu[T_Q]->insertItem("Resume &download", PM_ActivateGDL);
  menu[T_Q]->insertItem("Remove from &queue", PM_DeleteGDL);
  menu[T_Q]->insertSeparator();
  menu[T_Q]->insertItem("&Remove source", PM_CloseDL);
  menu[T_Q]->insertItem("Get user's &file list", PM_GetFileList);
  menu[T_UL] = new QPopupMenu(this, "UploadPopupMenu");
  menu[T_UL]->insertItem("&Close connection", PM_CloseUL);
  menu[T_UL]->insertItem("Get user's &file list", PM_GetFileList);
  menu[T_H] = new QPopupMenu(this, "HistoryPopupMenu");
  menu[T_H]->insertItem("&Clear history", PM_ClearHistory);  
  menu[T_H]->insertSeparator();
  menu[T_H]->insertItem("Get user's &file list", PM_GetFileList);

  // Connect signals/slots for menus
  for (int t=0; t<T_MAX; t++) {
    connect(lv[t], 
	    SIGNAL(rightButtonClicked(QListViewItem*, const QPoint&, int)),
	    SLOT(showMenu(QListViewItem*, const QPoint&, int)));
    connect(menu[t], 
	    SIGNAL(activated(int)),
	    SLOT(menuClicked(int)));
  }

  // Connect gdl-update timer to its slot in this object.
  gdlup_timer = new QTimer(this);
  QObject::connect(gdlup_timer, SIGNAL(timeout()), this, SLOT(trigger_gdl_update()));

  signal_emit_timer = new QTimer(this);
  QObject::connect(signal_emit_timer, SIGNAL(timeout()), this, SLOT(emit_queued_signal()));
  signal_emit_timer->start(SIGNAL_EMIT_INTERVAL, false);

  // Set up the timestamp for GDLs and download sources.
  update_gdl_end();
  trigger_gdl_update();
}



void transfer_widget::menuClicked(int i)
{
  int page = tabs->currentPageIndex();

  if (!lastClicked[page])
    return;

  // Set up variables.
  QString username;
  gdl_item *gdl = NULL;
  transfer_item *udl = NULL;

  if (page != T_H) {
    gdl = ((transfer_lvi*)lastClicked[page])->GDL();
    udl = ((transfer_lvi*)lastClicked[page])->UDL();
    if (udl) 
      username = udl->user_name;
    else 
      username = ""; // gdl
  } else
    username = lastClicked[page]->text(3); // history list item

  //
  // Emit appropriate signal.
  //
  switch(i) {
  case PM_CloseGDL:
    debugWin->print("PM_CloseGDL\n");
    emit close_gdl(gdl->id());
    trigger_gdl_update();
    break;


  case PM_CloseDL:
    debugWin->print("PM_CloseDL\n");
    emit close_dl(udl->id, username, udl->file_name);

    if (page==T_DL)
      udl->status = X_CLOSING;
    else {
      GDLsMap::iterator j;

      // Mark corresponding download in DL-tab as closing, if exists.  What a mess.
      j = active_gdls.find(udl->id);
      if (j!=active_gdls.end()) {
	list<transfer_item>::iterator k = (*j)->sources_first();
	bool done=false;
	while (k != (*j)->sources_last() && !done) {
	  if ((*k) == *udl) {
	    (*k).status = X_CLOSING;
	    done = true;
	  } else k++;
	}
      }

      // Mark queued-item source for removal
      udl->status = X_REMOVED;

      // Delete source in queue that was marked for removal, using the
      // parent gdl_item's method delete_removed().
      j = queued_gdls.find(udl->id);
      if (j!=queued_gdls.end())
	(*j)->delete_removed();
    }

    trigger_gdl_update();
    break;


  case PM_ActivateGDL:
    debugWin->print("PM_ActivateGDL\n");
    debugWin->print("This is not implemented and needs to be resolved.\n");
    break;

  case PM_DeleteGDL: {
    debugWin->print("PM_DeleteGDL\n");
    emit close_gdl(gdl->id());

    GDLsMap::iterator q = queued_gdls.find(gdl->id());
    if (q != queued_gdls.end()) {
      queued_gdls.remove(q);
    }

    delete gdl;
    trigger_gdl_update();
  }
    break;

  case PM_CloseUL:
    debugWin->print("PM_CloseUL\n");
    udl->status = X_CLOSING;
    emit close_ul(udl->id);
    break;

  case PM_ClearHistory:
    debugWin->print("PM_ClearHistory\n");
    // I'm not gonna traverse their siblings. Just kill and resurrect
    delete history_ul;
    delete history_dl;
    // History's upload & download root items.
    history_dl = new QListViewItem(lv[T_H], "Downloaded", "0", "0 Byte", "");
    history_dl->setExpandable(true);
    history_ul = new QListViewItem(lv[T_H], "Uploaded", "0", "0 Byte", "");
    history_ul->setExpandable(true);
    break;

  case PM_GetFileList:
    debugWin->print("PM_GetFileList\nUser: " + username + "\n");
    emit getUserFileList(username);
    break;

  default:
    debugWin->print("In transfer_widget::menuClicked: Unhandled popup menu item index!\n");
  }

}


void transfer_widget::showMenu(QListViewItem* i, const QPoint &p, int)
{
  if(i) {
    int page = tabs->currentPageIndex();

    QString uname;
    if (page!=T_H)
      uname = ((transfer_lvi *)i)->text(transfer_lvi::CT_UNAME);
    else
      uname = i->text(3);

    lastClicked[page] = (QListViewItem*)i;

    // Enable/disable appropriate menu choices depending on what kind
    // of item has been selected, and change menu text according to
    // contents of the item.
    switch(page) {

    case T_DL:
      if (((transfer_lvi *)i)->GDL()) {
	menu[page]->setItemEnabled(PM_CloseGDL, true);
	menu[page]->changeItem(PM_CloseDL, "&Close connection");
	menu[page]->setItemEnabled(PM_CloseDL, false);
	menu[page]->changeItem(PM_GetFileList, "Get user's &file list");
	menu[page]->setItemEnabled(PM_GetFileList, false);		
      } else {
	menu[page]->setItemEnabled(PM_CloseGDL, false);
	menu[page]->changeItem(PM_CloseDL, "&Close connection to " + uname);
	menu[page]->setItemEnabled(PM_CloseDL, true);
	menu[page]->changeItem(PM_GetFileList, "Get " + uname + "'s &file list");
	menu[page]->setItemEnabled(PM_GetFileList, true);		
      }
      break;

    case T_Q:
      if (((transfer_lvi *)i)->GDL()) {
	menu[page]->setItemEnabled(PM_ActivateGDL, false /*true*/); // this will be enabled when implemented. setting to false for now
	menu[page]->setItemEnabled(PM_DeleteGDL, true);
	menu[page]->changeItem(PM_CloseDL, "&Remove source");
	menu[page]->setItemEnabled(PM_CloseDL, false);
	menu[page]->changeItem(PM_GetFileList, "Get user's &file list");
	menu[page]->setItemEnabled(PM_GetFileList, false);		
      } else {
	menu[page]->setItemEnabled(PM_ActivateGDL, false);
	menu[page]->setItemEnabled(PM_DeleteGDL, false);
	menu[page]->changeItem(PM_CloseDL, "&Remove source " + uname);
	menu[page]->setItemEnabled(PM_CloseDL, true);
	menu[page]->changeItem(PM_GetFileList, "Get " + uname + "'s &file list");
	menu[page]->setItemEnabled(PM_GetFileList, true);		
      }      
      break;

    case T_UL:
      menu[page]->changeItem(PM_CloseUL, "&Close connection to " + uname);
      menu[page]->setItemEnabled(PM_CloseUL, true);
      menu[page]->changeItem(PM_GetFileList, "Get " + uname + "'s &file list");
      menu[page]->setItemEnabled(PM_GetFileList, true);		
      break;

    case T_H:
      if (i->isExpandable()) {
	menu[page]->setItemEnabled(PM_ClearHistory, true);
	menu[page]->changeItem(PM_GetFileList, "Get user's &file list");
	menu[page]->setItemEnabled(PM_GetFileList, false);		
      } else {
	menu[page]->setItemEnabled(PM_ClearHistory, true);
	menu[page]->changeItem(PM_GetFileList, "Get " + uname + "'s &file list");
	menu[page]->setItemEnabled(PM_GetFileList, true);		
      }
      break;

    default:
      return;
    }

    menu[page]->popup(p);
  }
}



bool transfer_widget::user_busy(const QString &uname) const
{
  GDLsMap::const_iterator i = queued_gdls.begin();
  while (i!=queued_gdls.end()) {
    list<transfer_item>::iterator k = (*i)->sources_first();
    while (k != (*i)->sources_last()) {
      if ((*k).user_name == uname)
	return true;
      else 
	k++;
    }
  }

  i = active_gdls.begin();
  while (i!=active_gdls.end()) {
    list<transfer_item>::iterator k = (*i)->sources_first();
    while (k != (*i)->sources_last()) {
      if ((*k).user_name == uname)
	return true;
      else 
	k++;
    }
  }

  return false;
}


void transfer_widget::restart_download(const QString &fname)
{
  gdl_item *gdl=NULL;

  // See if a GDL for this file is in the queue.
  GDLsMap::iterator qi = queued_gdls.begin();
  while (qi!=queued_gdls.end() && gdl==NULL)
    if ((*qi)->filename() != fname)
      qi++;
    else
      gdl = (*qi);

  if (gdl==NULL)
    return;

  // Destroy the GDL, if still running.
  emit close_gdl(gdl->id());

  // Re-add each source that we know about.
  list<transfer_item>::iterator i = gdl->sources_first();
  while (i != gdl->sources_last()) {
    if (i==gdl->sources_first())
      enqueue_signal(gdl->id(), (*i).user_name, (*i).file_name, gdl->filename(), (*i).file_size);
    else
      enqueue_signal(gdl->id(), (*i).user_name, (*i).file_name, (*i).file_size);
  }

}


//
// TO DO:  Check if user is busy and only emit signals if not.
//         Maintain list of busy/available users.
//
//         ANSWER: I dunno if we need this.  DCTC queues DLs anyway.
void transfer_widget::download_file(const QString &user, 
				    const QString &remote_file,
				    const QString &local_file, 
				    unsigned long int fsize)
{
  gdl_item *gdl=NULL;

  // See if this GDL is in the queue.
  GDLsMap::iterator qi = queued_gdls.begin();
  while (qi!=queued_gdls.end() && gdl==NULL)
    if ((*qi)->filename() != local_file || (*qi)->filesize()!=fsize)
      qi++;
    else
      gdl = (*qi);
  
  // If the GDL is not in the queue, add it.
  if (gdl==NULL) {
    // create storage item
    gdl = new gdl_item(local_file, fsize);

    // add it to the queue
    queued_gdls.insert(gdl->id(), gdl);

    // create view item for the GDL
    ((transfer_lvi*)new transfer_lvi(lv[T_Q]))->set_layout(transfer_lvi::Q_GDL, gdl);    

    // If we had to create a new GDL, we will emit 'start_gdl' signal.
    enqueue_signal(gdl->id(), user, remote_file, local_file, fsize);
  } else
    // If GDL already existed, we will emit 'add_gdl_source' signal.
    enqueue_signal(gdl->id(), user, remote_file, fsize);

  // Add user+file as a new source.
  transfer_item u(gdl->id(), user, remote_file);
  gdl->add_source(u);

  attach_view_items(gdl, transfer_lvi::Q_GDL_SRC);

  request_gdl_update();
}



void transfer_widget::attach_view_items(gdl_item *gdl, const int layout_type) 
{
  // Run through all sources and see if any need a view-item attached.
  list<transfer_item>::iterator src = gdl->sources_first();
  while (src!=gdl->sources_last()) {
    if ((*src).QLVI() == 0)
      ((transfer_lvi*)new transfer_lvi(gdl->QLVI()))->set_layout(layout_type, &(*src));
    src++;
  }  
}



QString transfer_widget::update_gdl_in_queue(GDLsMap &gdls,
					     QListView *tgt_lv,
					     const int layout_main_type,
					     const int layout_item_type,
					     const bool erase_outdated,
					     const unsigned int id,
					     const QString &fname,
					     const ULLINT fsize,
					     const ULLINT foffs,
					     const ULLINT frecv, 
					     const unsigned int time0,
					     const ULLINT rcv10,
					     const QString &running,
					     const QString &complete) 
{
  gdl_item *gdl=NULL;
  GDLsMap::iterator qi = gdls.find( id );

  if (qi != gdls.end()) {
    gdl = (*qi);
  } else {
    qi = gdls.begin();
    while (qi!=gdls.end() && gdl==NULL)
      if ((*qi)->filename() != fname || (*qi)->filesize()!=fsize)
	qi++;
      else
	gdl = (*qi);
  }

  // If the GDL is not in the queue, add it.
  if (gdl==NULL) {
    // create storage item
    gdl = new gdl_item( id, fname, fsize, foffs, frecv, time0, rcv10 );
    // add it to the queue
    gdls.insert(gdl->id(), gdl);
    // create view item
    ((transfer_lvi*)new transfer_lvi(tgt_lv))->set_layout(layout_main_type, gdl);    
  }

  // Now update the queued gdl item.
  gdl->update(gdlinfo_timestamp, fsize, foffs, frecv, time0, rcv10, running, complete);

  // Remove or Mark as queued all sources that haven't been updated.
  if (erase_outdated)
    gdl->remove_outdated(gdlinfo_timestamp);
  else
    gdl->mark_outdated(gdlinfo_timestamp);

  // Run through each source and see if any need view-items attached.
  attach_view_items(gdl, layout_item_type);

  return gdl->status_str();
}



void transfer_widget::update_gdl_info(const QString &info)
{

  // I know that this parsing stuff would look better if it was done
  // somewhere else, like, say, in transfer_storage.cc.  But it's here
  // now, and now I've got better things to do than shuffling chunks
  // of code around.

  unsigned int id = (info.section('|', 0,0)).toUInt();
  QString fname = info.section('|', 1,1);
  ULLINT fsize = (info.section('|', 2,2)).toULong();
  ULLINT foffs = (info.section('|', 3,3)).toULong();
  ULLINT frecv = (info.section('|', 4,4)).toULong();
  unsigned int time0 = (info.section('|', 5,5)).toUInt();
  ULLINT rcv10 = (info.section('|', 6,6)).toULong();

  QString running="", complete="";

  int i=10;
  while(info.section('|', i,i)!="") {
    running += info.section('|', i,i) + "|";
    i++;
  }
  running += "";

  i++;

  while(info.section('|', i,i)!="") {
    complete += info.section('|', i,i) + "|";
    i++;
  }
  complete += "";

  QString status = update_gdl_in_queue(queued_gdls, lv[T_Q], transfer_lvi::Q_GDL, transfer_lvi::Q_GDL_SRC, false,
				       id, fname, fsize, foffs, frecv, time0, rcv10, running, complete);

  if (status!="Queued" && status!="Closed" && status!="DCTC-Queued" && status!="Waiting")
    update_gdl_in_queue(active_gdls, lv[T_DL], transfer_lvi::A_GDL, transfer_lvi::A_GDL_SRC, true,
			id, fname, fsize, foffs, frecv, time0, rcv10, running, complete);
}



//
// activated when gdl info list ends
//
void transfer_widget::update_gdl_end()
{
  // First check any files that we thought could be complete in the previous iteration

  list<ULLINT>::iterator ci = completed_gdl_list.begin();
  while (ci != completed_gdl_list.end()) {

    // Find gdl by id.
    GDLsMap::iterator k = active_gdls.find((*ci)), j = queued_gdls.find((*ci));

    QString tgt_file = settings.get_setting(dc_settings::DLDIR) + (*k)->filename();
    QFileInfo fi(tgt_file);
    fi.refresh();

    // is the file download complete?  Skiter i storleken, it's enuff if the file exists in downloads!
    if (fi.exists()) { //  && fi.size() == (*k)->filesize()) {
      // if download is complete, move it to history.
      QString first_user = "";
      if ((*k)->sources_first() != (*k)->sources_last()) {
	list<transfer_item>::iterator l = (*k)->sources_first();
	first_user = (*l).user_name;
      }
      move_to_history(history_dl, (*k)->filename(), first_user, fi.size());

      // remove from queue if it's there
      if (j!=queued_gdls.end()) {
	queued_gdls.remove(j);
	delete (*j);
      }

    } else {
      // if not complete, just mark outdated sources in queue
      if (j!=queued_gdls.end())
	(*j)->mark_outdated(gdlinfo_timestamp);
    }

    // remove from running downloads, complete or not.
    active_gdls.remove(k);
    delete (*k);

    completed_gdl_list.pop_front();
    ci = completed_gdl_list.begin();
  }

  // Now check if there are any un-updated downloads and store their gdl id
  // in completed_gdl_list to check next iteration.
  GDLsMap::iterator i = active_gdls.begin();

  while (i!=active_gdls.end()) {
    // outdated?  store id in completed list
    if ((*i)->timestamp < gdlinfo_timestamp)
      completed_gdl_list.push_back((*i)->id());
    // move on to next
    i++;
  }

  for (int lvnum=0; lvnum<T_MAX; lvnum++)
    lv[lvnum]->triggerUpdate();

  // get next timestamp in milliseconds
  struct timeval tv;
  struct timezone tz;
  gettimeofday(&tv, &tz);
  
  // Set timestamp that we'll be using next iteration.
  gdlinfo_timestamp = tv.tv_usec;

  is_updating = false;

  gdlup_timer->start(GDL_UPDATE_INTERVAL, true);
}



//  
// new upload
//
void transfer_widget::new_ul_info(const QString &info)
{
  QString ulids = info.section('|', 0,0);
  QString uname = info.section('|', 1,1);
  QString fname = info.section('|', 2,2);
  bool cnvOk = false;

  // Only insert uploads of valid users&files, but not shared file list.
  if (ulids.isNull() || fname.isNull() || uname.isNull() || uname.find("Shared file list")!=-1)
    return;

  unsigned int ul_id = ulids.toUInt(&cnvOk);
  if (cnvOk) {
    transfer_item *ul;
    UploadsMap::iterator i = ul_map.find ( ul_id );
    
    if (i==ul_map.end()) {
      ul = new transfer_item(ul_id);
      ((transfer_lvi*)new transfer_lvi(lv[T_UL]))->set_layout(transfer_lvi::UL, ul);
      ul_map.insert(ul_id, ul);
      i = ul_map.find ( ul_id );
    } else {
      ul = (*i);
    }
    
    ul->user_name = uname; // info.section('|', 1, 1);

    char dbg[256];
    sprintf(dbg, "New upload to: \"%s\"", ul->user_name.ascii());
    debugWin->print(dbg);

    /* paths are not relative... but for some stupid reason there's an "UL/" infront of the path/filename...

      QString rest = info.section('|', 2, 2);
      int index = rest.findRev('/');
      ul->file_name = rest.remove(0,index+1);
    */

    if (fname.find("UL/")!=-1)
      ul->file_name = fname.remove("UL/");
    else
      ul->file_name = fname;

    ul->file_size = 0;
    ul->status = X_WAITING;
    ul->range = ull_pair(0,1);
    ul->bytes_done = 0;
  }
}



void transfer_widget::update_ul_info(const QString &info)
{
  // aaa:bbb/ccc/ddd/eee
  //     where aaa is an ID identifying the transfer (the same as in
  //     /XFERR), bbb is the upload start position, ccc is the total
  //     file length (not just the part to send), ddd is the amount of
  //     data already sent (not including bbb) and eee is number of
  //     bytes to send. Currently, we have the following relation:
  //     bbb+eee=ccc and 0<ddd<eee.

  unsigned int ul_id = (info.section(':', 0,0)).toUInt();

  transfer_item *ul;
  UploadsMap::iterator i = ul_map.find ( ul_id );

  if (i==ul_map.end()) 
    return;

  ul = *i;

  QString rest = info.section(':', 1,1);
  ul->file_size = (rest.section('/', 1,1)).toULong();
  ul->range = ull_pair((rest.section('/', 0,0)).toULong(), ul->file_size);
  ul->bytes_done = (rest.section('/', 2,2)).toULong();
  ul->status = X_UPLOADING;

  // UGLY! estr fix this!!  --rkrd
  static int ful_counter = 0;
  if(ful_counter==60)
  {
       ful_counter = 0;
       lv[T_UL]->triggerUpdate();
  }
  ful_counter++;
}



void transfer_widget::closed_ul_info(const QString &user, const QString &file)
{
  char dbg[256];

  if (user.isNull() || file.isNull())
    return;

  sprintf(dbg, "Closed upload: \"%s\"", file.ascii());
  debugWin->print(dbg);

  //  transfer_item *ul;
  UploadsMap::iterator i = ul_map.begin();
  
  while (i != ul_map.end()) {
    if ((*i)->user_name != user || (*i)->file_name != file)
      i++;
    else 
      break;
  }

  if (i == ul_map.end()) 
    return;

  move_to_history(history_ul, file, user, (*i)->file_size);
  
  delete (*i);
  lv[T_UL]->triggerUpdate();

  debugWin->print("Closed upload matched and moved to history.");
}




void transfer_widget::move_to_history(QListViewItem *h, const QString &file, const QString &user, const ULLINT size)
{
  (void) new QListViewItem(h, file, "", bytes_to_string(size), user);

  int i = (h->text(2)).findRev(' ');
  int factor=1;
  if ((h->text(2)).endsWith("KB"))
    factor = 1024;
  else
    if ((h->text(2)).endsWith("MB"))
      factor = 1024*1024;
    else
      if ((h->text(2)).endsWith("GB"))
	factor = 1024*1024*1024;

  ULLINT total = (ULLINT) (factor * ((h->text(2)).left(i)).toDouble()) + size;
  h->setText(2, bytes_to_string(total));

  // #files
  h->setText(1, QString::number((h->text(1).toUInt())+1));
}



void transfer_widget::trigger_gdl_update() 
{
  if (gdlup_timer->isActive())
    return;

  if (is_updating) {
    gdlup_timer->start(GDL_UPDATE_INTERVAL, true);
  } else {
    is_updating = true;
    emit request_gdl_update();
  }
}


void transfer_widget::enqueue_signal(ULLINT id, QString user, QString remote, QString local, ULLINT size)
{
  queued_signal s;
  s.is_new_gdl = true;
  s.id = id;
  s.user = user;
  s.remote_file = remote;
  s.local_file = local;
  s.size = size;
  dl_signal_queue.push_back(s);
}

void transfer_widget::enqueue_signal(ULLINT id, QString user, QString remote, ULLINT size)
{
  queued_signal s;
  s.is_new_gdl = false;
  s.id = id;
  s.user = user;
  s.remote_file = remote;
  s.size = size;
  dl_signal_queue.push_back(s);
}

void transfer_widget::emit_queued_signal()
{
  if (dl_signal_queue.size()>0) {
    queued_signal s = dl_signal_queue.front();
    dl_signal_queue.pop_front();
    
    if (s.is_new_gdl)
      emit start_gdl(s.id, s.user, s.remote_file, s.local_file, s.size);
    else
      emit add_gdl_source(s.id, s.user, s.remote_file, s.size);
  }
}


/*
 *
 * $Log: transfer_widget.cc,v $
 * Revision 1.11  2004/02/10 08:41:25  estrato
 * Fixed upload bug (for the 11th time)
 *
 * Revision 1.10  2004/01/27 16:09:44  estrato
 * little fix in upload updates
 *
 * Revision 1.9  2004/01/26 18:49:59  estrato
 * fixed upload bugs
 *
 * Revision 1.8  2003/12/08 00:13:40  estrato
 * Directory downloads
 *
 * Revision 1.7  2003/09/09 18:55:59  estrato
 * bugfix in handling of completed downloads
 *
 * Revision 1.6  2003/09/09 18:49:34  estrato
 * More stable transfer widget
 *
 * Revision 1.5  2003/08/22 16:45:20  olof
 * hola
 *
 * Revision 1.4  2003/08/20 15:56:26  estrato
 * Minor changes to transfer widget, fixed a bug with downloads from user file list.
 *
 * Revision 1.3  2003/08/18 12:19:43  estrato
 * Old transfer dialog now completely replaced, but some functionality is gone. Will add complete functionality in next commit.
 *
 * Revision 1.2  2003/08/16 21:34:19  estrato
 * New transfer dialog update, almost all internal functions done.  Will replaceold transferdlg in next commit.
 *
 * Revision 1.1  2003/08/14 23:30:45  estrato
 * New transfer widget, basic storage and view functionality, no signals connected yet.
 *
 *
 *
 */
