
/** 
  The way this will work (preliminary):
  ------------------------------------
 
  o A unique download is identified by a User-File pair.

  o A unique GDL (group of downloads) is indentified by a File and GDL ID.

  o A download is always stored as a unique source inside a GDL item.

  o A GDL item can contain multiple downloads: same file name, but
  different users.

  o Active downloads are stored inside the GDL in the active_gdls list,
  while downloads that are queued are stored inside the GDL item in
  queued_gdls list.

  o If a GDL has any active downloads, then a static copy of the same
  GDL also exists in the active_gdls list.  The copy has same File and
  ID, but unlike the original, it is required to contain at least one
  download (active).  If the last active download stops, the GDL copy
  is destroyed.

  o A GDL whose File isn't fully downloaded remains in the queued_gdls
  list. GDLs in the queued_gdls list do not have to contain any sources.
  
  o A GDL in queued_gdls is destroyed only when it's File is completely
  downloaded.  The completed File's name is then added to the download
  history.

  o If an inactive download becomes active, it is moved into the
  corresponding GDL in active_gdls list.  If no such GDL exists, it's
  created.

  o If a download finishes, but the GDL still has other downloads running,
  the download is destroyed.

  o If an active download is aborted abnormally, it is moved back to
  the master GDL in the queued_gdls list.

  o If you haven't slept for 36+ hours, you begin to automatically drool

*/


#ifndef _GDL_TRANSFER_WIDGET_H_
#define _GDL_TRANSFER_WIDGET_H_

#include <qtimer.h>
#include <qlistview.h>
#include <qtabwidget.h>
#include <qpopupmenu.h>     

#include <list>

#include "transfer_storage.h"

using namespace std;

class transfer_widget : public QWidget
{
  Q_OBJECT
  
protected:

  enum T_TABS {T_DL=0, 
	       T_Q, 
	       T_UL, 
	       T_H, 
	       T_MAX};

  enum PM_IDS {PM_CloseGDL=1, 
	       PM_ActivateGDL, 
	       PM_DeleteGDL, 
	       PM_CloseDL, 
	       PM_CloseUL, 
	       PM_ClearHistory, 
	       PM_GetFileList};

  // Start dl/gdl signals are not emitted at once, but put in a queue.
  // A timer-based function pops queued signals and emits them one at
  // a time.  This struct is what we use to store signal data in.
  struct queued_signal {
    bool is_new_gdl;
    ULLINT id;
    QString user;
    QString remote_file;
    QString local_file;
    ULLINT size;
  };

  // Queue of new gdl/dl signals.
  list<queued_signal> dl_signal_queue;

  // List of download files that may have been completed.  DCTC may
  // need a short time to finish renaming a complete file after it no
  // longer appears in gdl info list, so we store possibly completed
  // files here for a while before we check them.
  list <ULLINT> completed_gdl_list;

  // GDL-update timestamp
  unsigned int gdlinfo_timestamp;

  // GDL-update-request signal timer
  QTimer *gdlup_timer;
  bool is_updating;

  // Timer for emitting signal Start GLD / Add GDL source
  QTimer *signal_emit_timer;
  
  // Store GDLs (queued & active) and uploads in maps
  typedef QMap<unsigned int, transfer_item *> UploadsMap;
  typedef QMap<unsigned int, gdl_item *> GDLsMap;

  UploadsMap ul_map;
  GDLsMap queued_gdls;
  GDLsMap active_gdls;

  // Download and upload history top-level items
  QListViewItem *history_dl;
  QListViewItem *history_ul;

  // names of users from whom we are downloading. 
  list<QString> connected_users;

  // GUI components
  QTabWidget *tabs;
  QListView  *lv[T_MAX];
  QPopupMenu *menu[T_MAX];
  QListViewItem *lastClicked[T_MAX];

protected:

  bool user_busy(const QString &uname) const;  // deprecated

  // Using these two methods to queue download-start signals.
  void enqueue_signal(ULLINT id, QString user, QString remote, QString local, ULLINT size);
  void enqueue_signal(ULLINT id, QString user, QString remote, ULLINT size);

  void attach_view_items(gdl_item *gdl, const int layout_type);
  
  QString update_gdl_in_queue(GDLsMap &gdls,
			      QListView *tgt_lv,
			      const int layout_main_type,
			      const int layout_item_type,
			      const bool erase_outdated,
			      const unsigned int id,
			      const QString &fname,
			      const ULLINT fsize,
			      const ULLINT foffs,
			      const ULLINT frecv, 
			      const unsigned int time0,
			      const ULLINT rcv10,
			      const QString &running,
			      const QString &complete);

  void move_to_history(QListViewItem *h, const QString &file, const QString &user, const ULLINT size);

public:

  transfer_widget( QWidget* parent = 0, const char* name = 0 );

  
public slots:
 
  // popup menus
  void menuClicked(int i);
  void showMenu(QListViewItem* i, const QPoint &p, int);
  
  // update info on a GDL
  void update_gdl_info(const QString &info);

  // no more gdl's updated this iteration
  void update_gdl_end();

  // emits "request for gdl info list" signal, avoids multiple unnecessary signals
  void trigger_gdl_update();

  // dequeues and emits a queued download-file signal (either new gld or new dl source)
  void emit_queued_signal();

  // receive info on a new upload
  void new_ul_info(const QString &info);
  // update info on an upload
  void update_ul_info(const QString &info);
  // update info on an upload
  void closed_ul_info(const QString &user, const QString &file);

  // Attaches to a broken download.
  void restart_download(const QString &fname);

  // (user, remote_file) identifies a source
  // (local_file, size) identifies a GDL
  void download_file(const QString &user, const QString &remote_file, const QString &local_file, unsigned long int fsize);

signals:

  void start_gdl(int id, const QString &user, const QString &remote_file, const QString &local_file, unsigned long int size);
  void add_gdl_source(int id, const QString &user, const QString &remote_file, unsigned long int size);

  // Attach to a broken/detached GDL in the GDL/ directory.
  void attach_gdl(const QString &fname);

  // at completion, rename gdl with this id to these fpath/fname
  void rename_gdl(unsigned int id, const QString &fname, const QString &fpath);

  void close_gdl(unsigned int id); // GDLEND, total destruction of GDL
  void close_dl(unsigned int id, const QString &user, const QString &remote_file);  // GDLDEL, kills source
  void close_ul(unsigned int id); // KILL, kill a transfer (we use it for uploads)

  void request_gdl_update();

  void getUserFileList(const QString& user);
  
};


#endif

/*
 *
 * $Log: transfer_widget.h,v $
 * Revision 1.5  2003/09/09 18:49:34  estrato
 * More stable transfer widget
 *
 * Revision 1.4  2003/08/20 15:56:26  estrato
 * Minor changes to transfer widget, fixed a bug with downloads from user file list.
 *
 * Revision 1.3  2003/08/18 12:19:43  estrato
 * Old transfer dialog now completely replaced, but some functionality is gone. Will add complete functionality in next commit.
 *
 * Revision 1.2  2003/08/16 21:34:19  estrato
 * New transfer dialog update, almost all internal functions done.  Will replaceold transferdlg in next commit.
 *
 * Revision 1.1  2003/08/14 23:30:45  estrato
 * New transfer widget, basic storage and view functionality, no signals connected yet.
 *
 *
 *
 */
